<?php

/**
 * @file
 * Configuration options for the module.
 */

/**
 * Administration form.
 */
function conditional_message_config_form($form, &$form_state) {

  $form['conditional_message'] = array(
    '#title' => t('Conditional message configurations'),
    '#type' => 'fieldset',
  );

  $conditional_message_options = array(
    'session' => t('once per session'),
    'role' => t('to certain user roles'),
    'path' => t('on certain paths'),
    'content_type' => t('on certain content types'),
  );

  $form['conditional_message']['conditional_message_options'] = array(
    '#title' => t('Conditions'),
    '#type' => 'checkboxes',
    '#description' => t('Select the conditions in which the message will be displayed. Several conditions can be selected.'),
    '#default_value' => variable_get('conditional_message_options', NULL),
    '#required' => FALSE,
    '#options' => $conditional_message_options,
    '#attributes' => array('class' => array('conditional-message-options-columns')),
  );

  // Display in columns.
  drupal_add_css('.conditional-message-options-columns {column-count:2;column-gap:10px;}', 'inline');

  $form['conditional_message']['conditional_message_text'] = array(
    '#title' => t('Message'),
    '#type' => 'textfield',
    '#description' => t("Set the message that will be displayed to users when the conditions are met (uses 'filtered html' filter and only &lt;a&gt; and &lt;em&gt; tags are allowed)."),
    '#default_value' => variable_get('conditional_message_text', 'Enter a custom message at /admin/config/user-interface/conditional-message'),
    '#size' => 100,
    '#maxlength' => 255,
    '#required' => TRUE,
  );

  $form['conditional_message']['conditional_message_options_config'] = array(
    '#title' => t('Conditions configurations'),
    '#type' => 'fieldset',
    '#states' => array(
      'visible' => array(
        array(
          array(':input[name="conditional_message_options[session]"]' => array('checked' => TRUE)),
          array(':input[name="conditional_message_options[role]"]' => array('checked' => TRUE)),
          array(':input[name="conditional_message_options[path]"]' => array('checked' => TRUE)),
          array(':input[name="conditional_message_options[content_type]"]' => array('checked' => TRUE)),
        )
      ),
    ),
  );

  $form['conditional_message']['conditional_message_options_config']['conditional_message_session'] = array(
    '#type' => 'markup',
    '#markup' => t('Mind that sessions (localStorage) persist even after closing '
        . ' a tab or window. For testing purposes the "session" can be cleared '
        . 'on a browser by typing the following command on the console: '
        . '"localStorage.removeItem(\'conditionalMessageReadStatus\');"'),
    '#states' => array(
      'visible' => array(
        ':input[name="conditional_message_options[session]"]' => array('checked' => TRUE),
      ),
    ),
  );

  $form['conditional_message']['conditional_message_options_config']['conditional_message_user_role'] = array(
    '#type' => 'checkboxes',
    '#title' => t('User roles'),
    '#description' => t('Select roles that will see the message'),
    '#options' => user_roles(),
    '#default_value' => variable_get('conditional_message_user_role'),
    '#required' => FALSE,
    '#states' => array(
      'visible' => array(
        ':input[name="conditional_message_options[role]"]' => array('checked' => TRUE),
      ),
    ),
  );

  $form['conditional_message']['conditional_message_options_config']['conditional_message_path'] = array(
    '#type' => 'textarea',
    '#title' => t('Paths'),
    '#description' => t('Enter one path per line. Aliases are not considered as equivalent so node/1 and first-page are considered different.'),
    '#default_value' => variable_get('conditional_message_path'),
    '#required' => FALSE,
    '#states' => array(
      'visible' => array(
        ':input[name="conditional_message_options[path]"]' => array('checked' => TRUE),
      ),
    ),
  );

  $form['conditional_message']['conditional_message_options_config']['conditional_message_content_type'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Content types'),
    '#description' => t('Select content types that will trigger the message'),
    '#options' => node_type_get_names(),
    '#default_value' => variable_get('conditional_message_content_type', array('article', 'page')),
    '#required' => FALSE,
    '#states' => array(
      'visible' => array(
        ':input[name="conditional_message_options[content_type]"]' => array('checked' => TRUE),
      ),
    ),
  );

  $form['conditional_message']['conditional_message_layout'] = array(
    '#title' => t('Message configurations'),
    '#type' => 'fieldset',
  );

  $form['conditional_message']['conditional_message_layout']['conditional_message_bg_colour'] = array(
    '#title' => t('Backgroud colour'),
    '#type' => 'textfield',
    '#field_prefix' => '#',
    '#description' => 'Enter a colour for the message background in HEX value.',
    '#default_value' => variable_get('conditional_message_bg_colour', 'CCCCCC'),
    '#size' => 6,
    '#maxlength' => 6,
    '#required' => FALSE,
  );

  $form['conditional_message']['conditional_message_layout']['conditional_message_colour'] = array(
    '#title' => t('Font colour'),
    '#type' => 'textfield',
    '#field_prefix' => '#',
    '#description' => 'Enter a colour for the message text in HEX value.',
    '#default_value' => variable_get('conditional_message_colour', '119966'),
    '#size' => 6,
    '#maxlength' => 6,
    '#required' => FALSE,
  );

  $position = array('top' => t('Top'), 'bottom' => t('Bottom'));
  $form['conditional_message']['conditional_message_layout']['conditional_message_position'] = array(
    '#type' => 'radios',
    '#title' => t('Message position'),
    '#default_value' => variable_get('conditional_message_position', 'top'),
    '#options' => $position,
    '#description' => t('Select the desired position for the conditional message.'),
  );

  $form['conditional_message']['conditional_message_layout']['conditional_message_target'] = array(
    '#title' => t('Target HTML container'),
    '#type' => 'textfield',
    '#description' => t('Enter an id or class of the container where the message will be appended. Default is "#page-wrapper".'),
    '#default_value' => variable_get('conditional_message_target', '#page-wrapper'),
    '#required' => TRUE,
  );

  $form['#validate'][] = 'conditional_message_config_form_validate';

  return system_settings_form($form);
}

/**
 * Validation of the conditional message configuration form.
 */
function conditional_message_config_form_validate($form, &$form_state) {

  // Verify if at least one condition is checked.
  $error = TRUE;
  foreach ($form_state['values']['conditional_message_options'] as $option) {
    if ($option !== 0) {
      $error = FALSE;
    }
  }
  if ($error) {
    drupal_set_message('There is no condition selected to trigger a message, consider disabling this module.', 'warning');
  }

  // Apply filtered_html filter and strip all tags that are not allowed.
  $raw = check_markup($form_state['values']['conditional_message_text'], 'filtered_html');
  $allowed = '<a><em>';
  $form_state['values']['conditional_message_text'] = strip_tags($raw, $allowed);

  // Button to reset table of users that have seen the message.
  if (strstr($form_state['values']['op'], 'registered users have seen the message. Click to reset.')) {
    // Truncate table of registered users that have seen the message.
    $query = db_truncate('conditional_message_uid')->execute();
  }

  // Check roles configurations, at least one selected.
  if ($form_state['values']['conditional_message_options']['role'] !== 0) {
    $error = TRUE;
    foreach ($form_state['values']['conditional_message_user_role'] as $value) {
      if ($value !== 0) {
        $error = FALSE;
      }
    }
    if ($error) {
      form_set_error('conditional_message_user_role', 'At least one role must be selected if you chose to use the role as a condition.');
    }
  }

  // Check path configuration consistency.
  if ($form_state['values']['conditional_message_options']['path'] !== 0) {
    if (empty(trim(str_replace(PHP_EOL, '', $form_state['values']['conditional_message_path'])))) {
      form_set_error('conditonal_message_path', 'Enter at least one path if you chose to use the path as a condition.');
    }
    // Check if the specified paths exists and are accessible.
    $paths = explode(PHP_EOL, $form_state['values']['conditional_message_path']);
    foreach ($paths as $path) {
      // Empty lines are tolerated so we check if path is not empty.
      if (!empty(trim(str_replace(PHP_EOL, '', $path))) && !drupal_valid_path(trim($path))) {
        form_set_error('conditonal_message_path', 'All paths must exist and you must have permission to access it.');
      }
    }
  }

  // Check content type config consistency.
  if ($form_state['values']['conditional_message_options']['content_type'] !== 0) {
    $error = TRUE;
    foreach ($form_state['values']['conditional_message_content_type'] as $value) {
      if ($value !== 0) {
        $error = FALSE;
      }
    }
    if ($error) {
      form_set_error('conditional_message_content_type', 'At least one content type must be selected if you chose to use the content type as a condition.');
    }
  }

  // Check for valid HEX colour code.
  $c = $form_state['values']['conditional_message_colour'];
  if (!ctype_xdigit($c) || (drupal_strlen($c) !== 6 && drupal_strlen($c) !== 3)) {
    form_set_error('conditional_message_colour', 'Not a valid HEX colour. Please enter a 3 or 6 HEX digit on the Background colour field.');
  }
}
